{ :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  :: QuickReport 2.0 for Delphi 1.0/2.0/3.0                  ::
  ::                                                         ::
  :: Example QRCreateList() project (32 bit)                 ::
  :: Last update: March 8th, 1998                            ::
  ::                                                         ::
  :: Copyright (c) 1998 QuSoft AS                            ::
  :: All Rights Reserved                                     ::
  ::                                                         ::
  :: web: http://www.qusoft.com  fax: +47 22 41 74 91        ::
  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: }
unit main32;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DBCtrls, Grids, DBGrids, DBTables, Db, ExtCtrls, StdCtrls,
  quickrpt, qrextra, qrprntr, qrctrls;

type
  TfrmGrid32 = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    dstbCountry: TDataSource;
    DBGrid1: TDBGrid;
    DBNavigator1: TDBNavigator;
    rgRecords: TRadioGroup;
    btnPreview: TButton;
    GroupBoxFields: TGroupBox;
    rbAllFields: TRadioButton;
    rbSelectFields: TRadioButton;
    ListBoxFields: TListBox;
    GroupBoxSummary: TGroupBox;
    cBoxArea: TCheckBox;
    cBoxPopulation: TCheckBox;
    qryCountry: TQuery;
    qryCountryName: TStringField;
    qryCountryCapital: TStringField;
    qryCountryContinent: TStringField;
    qryCountryArea: TFloatField;
    qryCountryPopulation: TFloatField;
    GroupBoxGroups: TGroupBox;
    cBoxGroupByContininent: TCheckBox;
    Panel3: TPanel;
    Memo1: TMemo;
    procedure btnPreviewClick(Sender: TObject);
    procedure rbAllFieldsClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure rgRecordsClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmGrid32: TfrmGrid32;

implementation

{$R *.DFM}

{ dup function from qrexta, used to guess the maximum width of a field }

procedure TfrmGrid32.rbAllFieldsClick(Sender: TObject);
begin
  {
  If you click one field, clear the other.  This emulates the radiogroup
  control, but gives finer placment over the items
  }

  { disable this event so it doesn't get stuck in an endless loop }
  rbAllFields.OnClick := nil;
  rbSelectFields.OnClick := rbAllFields.OnClick;

  if Sender = rbAllFields then
    rbSelectFields.Checked := not rbAllFields.Checked
  else
    rbAllFields.Checked := not rbSelectFields.Checked;

  rbAllFields.OnClick := rbAllFieldsClick;
  rbSelectFields.OnClick := rbAllFields.OnClick;

  { We only enable the field list when the rbSelectFields button is set }
  ListBoxFields.Enabled := rbSelectFields.Checked;
end;

procedure TfrmGrid32.FormCreate(Sender: TObject);
var
  nIdx: integer;
begin
  { Prefill the listbox with the list of fields from the table }
  with qryCountry,ListBoxFields do
    for nIdx := 0 to FieldCount - 1 do
      Items.Add(Fields[nIdx].FieldName);

  { Call the field selection event to initialize the values}
  rbAllFieldsClick(rbAllFields);
end;

procedure TfrmGrid32.rgRecordsClick(Sender: TObject);
begin
  {
  We only allow summary information when more than one record is being
  output
  }
  GroupBoxSummary.Enabled := rgRecords.ItemIndex = 0;
  cBoxArea.Enabled := GroupBoxSummary.Enabled;
  cBoxPopulation.Enabled := GroupBoxSummary.Enabled;
end;

function dup(aChar : Char; Count : integer) : string;
var
  I : integer;
begin
  result := '';
  for I := 1 to Count do result := result + aChar;
end;

procedure TfrmGrid32.btnPreviewClick(Sender: TObject);
var
  aReport: TQuickRep;
  QRGroup1: TQRGroup;
  QRGroupFooter: TQRBand;
  FieldList: TStringList;
  nIdx: integer;
  SumArea, SumPopulation,
  SumGroupArea, SumGroupPopulation: TQRExpr;
begin
{
   We can pass in a list of dataset fields to include on the report.  If the
   list is nil or contains 0 items, all of the fields will be used.
}

  { default the list to nil }
  FieldList := nil;

  { default the group footer to nil }
  QRGroupFooter := nil;
  SumArea := nil;
  SumGroupArea := nil;

{
  Check to see if the user has slected any fields and add the selected
  fields to the list
}
  with ListBoxFields do
    if Enabled then
    begin
      FieldList := TStringList.Create;
      for nIdx := 0 to Items.Count - 1 do
        if Selected[nIdx] then
          FieldList.Add(Items[nIdx]);
    end;

{
  Make sure the new report object is set to nil, otherwise the
  QRCreateList function will not create it and you will get an access
  violation
}
  aReport := nil;

{
  Make the call to create the report.  This will create the report with
  a Column Header band and a Detail band.  You can customize the report
  before you call preview or print.
}
  QRCreateList(aReport, Self, qryCountry, 'Country Report', FieldList);

  { Here's where we customize the report...  }

  {
    Here's how to add a group band at runtime
  }
  if cBoxGroupByContininent.Checked then
  begin
    { First, create the group band }
    QRGroup1 := TQRGroup.Create(Self);

    { Now, set the properties for the group }
    with QRGroup1 do
    begin
      { assign it to this report }
      Parent := areport;

      { assign it to the detail band }
      Master := Parent;

      { Set it's expression }
      Expression := 'Continent';
    end;

    {
    Now add a text control to this group band and set it's properties
    }
    with TQRDBText.Create(Self) do
    begin
      Parent := QRGroup1;
      Dataset := areport.Dataset;
      DataField := 'Continent';
      Font.Style := [fsItalic, fsBold];
      Top := Parent.Height - Height;
    end;
    { Now create a group footer }
    QRGroupFooter := TQRBand.Create(Self);
    with QRGroupFooter do
    begin
      Parent := areport;
      BandType := rbGroupFooter;
    end;
    QRGroup1.FooterBand := QRGroupFooter;
  end;

{
  If the user has selected all of the records, we check to see if summary
  information is desired.  I have added some simple code to do a summary
  band with a couple of SUM()s.
}
  if rgRecords.ItemIndex = 0 then
  begin
    if cBoxArea.Checked or cBoxPopulation.Checked then
    begin
      with aReport do
      begin
        { Quick and easy way to add a summary band }
        Bands.HasSummary := True;

        { Make it bigger }
        Bands.SummaryBand.Height := round(Bands.SummaryBand.Height * 2.0);

        { User has requested to sum the area field }
        if cBoxArea.Checked then
        begin
          {
          We create a TQRLabel and a TQRExr control for the summary.  We
          borrow some of the code from the qrextra unit to calculate the
          width of controls.
          }
          SumArea := TQRExpr(Bands.SummaryBand.AddPrintable(TQRExpr));

          with SumArea do
          begin
            AutoSize := false;
            with TQRLabel(Bands.SummaryBand.AddPrintable(TQRLabel)) do
            begin
              Parent := Bands.SummaryBand;
              Top := 20;
              { Simple code for getting the maximum width of the field }
              AutoSize := true;
              Caption := Dup('X', qryCountryArea.DisplayWidth);
              AutoSize := false;
              SumArea.Width := Width;
              AutoSize := True;

              { Set the caption of the summary field}
              Caption := 'Total Area';

              { Set the font and bottom frameto make it stand out }
              Font.Style := [fsBold];
              Frame.DrawBottom := true;

              {
              Check to see if the caption is wider than the text and
              adjust the field widths either way
              }
              if SumArea.Width < Width then
                SumArea.Width := Width
              else
              begin
                AutoSize := false;
                Width := SumArea.Width;
              end;
              { Position the TQRExpr to be just under the label }
              SumArea.Top := Top + Height + 4;

              { Set the horizontal position }
              Left := 0;
              SumArea.Left := Left;
            end;
            { Grab the alignment from the dataset field }
            Alignment := qryCountryArea.Alignment;
            { Set the expression }
            Expression := 'SUM(Area)';
            { set the mask so that numbers are more readable }
            Mask := '#,';
          end;

          { If we have a group band, include sum on group footer }
          if QRGroupFooter <> nil then
          begin
            SumGroupArea := TQRExpr(QRGroupFooter.AddPrintable(TQRExpr));

            with SumGroupArea do
            begin
              AutoSize := false;
              Width := SumArea.Width;
              with TQRLabel(QRGroupFooter.AddPrintable(TQRLabel)) do
              begin
                Parent := QRGroupFooter;
                Top := 4;
                AutoSize := True;
                Caption := 'Total Continent Area';
                Font.Style := [fsBold];
                Frame.DrawBottom := true;

                if SumGroupArea.Width < Width then
                  SumGroupArea.Width := Width
                else
                begin
                  AutoSize := false;
                  Width := SumGroupArea.Width;
                end;

                SumGroupArea.Top := Top + Height + 4;
                { Set the horizontal position }
                left := SumArea.Left;
              end;
              left := SumArea.Left;
              Alignment := SumArea.Alignment;
              Expression := SumArea.Expression;
              Mask := SumArea.Mask;
              
              // reset the sum after printing each group
              ResetAfterPrint := True;
            end;
          end;
        end;

        if cBoxPopulation.Checked then
        begin
          SumPopulation := TQRExpr(Bands.SummaryBand.AddPrintable(TQRExpr));

          with SumPopulation do
          begin
            AutoSize := false;
            with TQRLabel(Bands.SummaryBand.AddPrintable(TQRLabel)) do
            begin
              Parent := Bands.SummaryBand;
              Top := 20;
              AutoSize := true;
              Caption := Dup('X', qryCountryPopulation.DisplayWidth);
              AutoSize := false;
              SumPopulation.Width := Width;
              AutoSize := True;
              Caption := 'Total Population';
              Font.Style := [fsBold];
              Frame.DrawBottom := true;
              if SumPopulation.Width < Width then
                SumPopulation.Width := Width
              else
              begin
                AutoSize := false;
                Width := SumPopulation.Width;
              end;

              SumPopulation.Top := Top + Height + 4;
              Left := 100;
              if SumArea <> nil then
                Left := SumArea.Left + SumArea.Width + 30
              else
                Left := 0;
              SumPopulation.Left := Left;
            end;
            Expression := 'SUM(Population)';
            Alignment := qryCountryPopulation.Alignment;
            Mask := '#,';
          end;

         { If we have a group band, include sum on group footer }
          if QRGroupFooter <> nil then
          begin
            SumGroupPopulation := TQRExpr(QRGroupFooter.AddPrintable(TQRExpr));

            with SumGroupPopulation do
            begin
              AutoSize := false;
              Width := SumPopulation.Width;
              with TQRLabel(QRGroupFooter.AddPrintable(TQRLabel)) do
              begin
                Parent := QRGroupFooter;
                Top := 4;
                AutoSize := True;
                Caption := 'Total Continent Population';
                Font.Style := [fsBold];
                Frame.DrawBottom := true;

                if SumGroupPopulation.Width < Width then
                  SumGroupPopulation.Width := Width
                else
                begin
                  AutoSize := false;
                  Width := SumGroupPopulation.Width;
                end;

                SumGroupPopulation.Top := Top + Height + 4;
                { Set the horizontal position }
                if SumGroupArea <> nil then
                  Left := SumGroupArea.Left + SumGroupArea.Width + 30
                else
                  Left := 0;

                SumGroupPopulation.Left := Left;
              end;
              Alignment := SumPopulation.Alignment;
              Expression := SumPopulation.Expression;
              Mask := SumPopulation.Mask;

              // reset the sum after printing each group
              ResetAfterPrint := True;
            end;
          end;
        end;
      end;
    end;
  end;
  if rgRecords.ItemIndex = 1 then
  begin
    with aReport do
    begin
      {
        With QuickReport 2, if you set PrintIfEmpty to True and the dataset
        property is set to nil, the column header band will not print.  By
        changing the band types around, we can force the bands to print.
      }
      Bands.TitleBand.BandType := rbPageHeader;
      Bands.ColumnHeaderBand.BandType := rbTitle;
      dataset := nil;
      PrintIfEmpty := true;
    end;
  end;

  {
  Loop through the controls on the detail band and set the format
  mask to display "," in the numbers.
  }
  with areport.bands.detailband do
    for nIdx := 0 to ControlCount - 1 do
      if Controls[nIdx] is TQRExpr then
        with TQRExpr(Controls[nIdx]) do
          if Alignment = taRightJustify then
            Mask := '#,';

  aReport.Preview;
  aReport.Free;

  {
  If we created a list of fields, then we free it when we are done with it.
  }
  if FieldList <> nil then
    FieldList.Free;
end;

end.
